/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.io.File;
import java.sql.Connection;

import org.databene.commons.StringUtil;
import org.databene.commons.iterator.HeavyweightIteratorProxy;
import org.databene.commons.iterator.TableRowIterator;
import org.databene.dbsanity.report.ReportUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Represents a sanity check composed of a test name and a SQL query which is assumed to 
 * return an empty result set. Any data row accessible from the iterator returned by 
 * the perform() method is interpreted as one faulty data set.<br/><br/>
 * Created: 13.10.2010 13:07:08
 * @since 1.0
 * @author Volker Bergmann
 */
public abstract class SanityCheck {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(SanityCheck.class);
	
	// configuration attributes
	protected String name;
	protected String description;
	protected String tableName;
	private String defectType;
	private String author;

	private File sourceFile;
	private File tmpResultFile;
	private File reportFolder;
	private File docPage;
	private File defectsPage;
	
	// result attributes
	protected int executionTime;
	protected long defectCounter;
	protected SanityCheckVerdict verdict;

	public SanityCheck(String name, String table, String defectType, File sourceFile, File tempFolder, File reportFolder, File docPage) {
		// check preconditions
		if (StringUtil.isEmpty(name))
			throw new IllegalArgumentException("No name specified for " + getClass().getSimpleName());
		if (table == null)
			LOGGER.warn("No table specified for " + getClass().getSimpleName() + " '" + name + "'");
		if (sourceFile == null) // TODO remove
			throw new IllegalArgumentException("No source file specified for " + getClass().getSimpleName());
		
		// initialize attributes
	    this.name = name;
	    this.tableName = table;
	    this.defectType = defectType;
	    this.reportFolder = reportFolder;
	    this.docPage = docPage;
	    this.defectsPage = new File(reportFolder, ReportUtil.normalizeUrl(name + ".defects.html"));
	    this.sourceFile = sourceFile;
	    this.tmpResultFile = new File(tempFolder, ReportUtil.normalizeUrl(getName()) + ".result");
		if (tmpResultFile != null && tmpResultFile.exists())
			tmpResultFile.delete();
		this.executionTime = 0;
	    this.defectCounter = 0;
	    this.verdict = SanityCheckVerdict.PENDING;
    }
	
	// properties --------------------------------------------------------------

	public String getName() {
    	return name;
    }

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}
	
	public String getTable() {
    	return tableName;
    }
	
	public void setTable(String tableName) {
		this.tableName = tableName;
	}
	
	public String getDefectType() {
		return defectType;
	}

	public File getSourceFile() {
		return sourceFile;
	}
	
	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public File getReportFolder() {
	    return reportFolder;
    }

	public File getDocPage() {
		return docPage;
	}
	
	public File getDefectsOrErrorPage() {
		return defectsPage;
	}
	
	public File getTmpResultFile() {
    	return tmpResultFile;
    }

	public SanityCheckVerdict getVerdict() {
		return verdict;
	}
	
    public int getExecutionTime() {
	    return executionTime;
    }

	public void setExecutionTime(int executionTime) {
		this.executionTime = executionTime;
	}
	
	public long getDefectCount() {
		return defectCounter;
	}

	// functional interface ----------------------------------------------------
	
	public abstract String getDetails();

	public abstract TableRowIterator query(Connection connection);
	
	public void skip() {
		this.verdict = SanityCheckVerdict.SKIPPED;
	}
	
	// java.lang.Object overrides ----------------------------------------------
	
	@Override
	public String toString() {
	    return name;
	}
	
	@Override
    public int hashCode() {
	    return name.hashCode();
    }

	@Override
    public boolean equals(Object obj) {
	    if (this == obj)
		    return true;
	    if (obj == null || getClass() != obj.getClass())
		    return false;
	    SanityCheck that = (SanityCheck) obj;
	    return this.name.equals(that.name);
    }

	// helper classes ----------------------------------------------------------
	
	public class Proxy extends HeavyweightIteratorProxy<Object[]> implements TableRowIterator {

		public Proxy(TableRowIterator source) {
	        super(source);
        }
		
		@Override
		public Object[] next() {
			// first query the source, then update the counter
			Object[] result = super.next();
			// thus, if there is an exception in the source's next() method, the counter is not increased
			defectCounter++;
			return result;
		}

		public String[] getColumnLabels() {
	        return ((TableRowIterator) source).getColumnLabels();
        }
		
	}

}
