package org.databene.dbsanity.model;

import java.io.File;
import java.sql.Connection;
import java.util.Date;
import java.util.Map;

import org.databene.commons.ConfigurationError;
import org.databene.commons.IOUtil;
import org.databene.commons.collection.OrderedNameMap;
import org.databene.commons.iterator.TableRowIterator;
import org.databene.dbsanity.Reporter;
import org.databene.dbsanity.report.ReportContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Combines atomic sanity checks to a composite sanity check suite.
 * @since 1.0
 * @author Yibo Wang
 * @author Volker Bergmann
 */
public class SanityCheckFile extends SanityCheckSuite {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(SanityCheckSuite.class);
	
	private File tmpResultFile;
	private File docPage;

	protected Map<String, SanityCheck> checks;
	private Date startDate;

	private File sourceFile;

	public SanityCheckFile(File sourceFile, File reportFolder, File tmpResultFile, File docPage, SanityCheckFolder parent) {
		super(sourceFile.getName(), reportFolder, parent);
		this.sourceFile = sourceFile;
		this.tmpResultFile = tmpResultFile;
		this.docPage = docPage;
		this.checks = new OrderedNameMap<SanityCheck>();
	}

	public File getSourceFile() {
		return sourceFile;
	}

	public void addCheck(SanityCheck check) {
		String checkName = check.getName();
		if (checks.containsKey(checkName))
			throw new ConfigurationError("Duplicate sanity check name: " + checkName);
	    this.checks.put(checkName, check);
    }

	public Map<String, SanityCheck> getChecks() {
    	return checks;
    }

	public File getTempFolder() {
		return tmpResultFile;
	}

	public File getDocPage() {
		return docPage;
	}

	@Override
    public void perform(Connection connection, Reporter reporter, ReportContext context) {
		startDate = new Date();
	    for (SanityCheck check : checks.values()) {
	    	if (context.acceptsTable(check.getTable())) {
	    		LOGGER.info("Performing check '{}'", check.getName());
	    		long start = System.nanoTime();
		    	TableRowIterator errorIterator = null;
		    	try {
			    	errorIterator = check.query(connection);
			    	reporter.report(check, errorIterator);
		    	} catch (Exception e) {
			    	reporter.error(check, e);
		    	} finally {
			    	int elapsed = (int) ((System.nanoTime() - start) / 1000000);
			    	check.setExecutionTime(elapsed);
		    		IOUtil.close(errorIterator);
		    	}
	    	} else {
	    		LOGGER.info("Skipping check '{}'", check.getName());
	    		check.skip();
	    		reporter.skipped(check);
	    		continue;
	    	}
	    }
	}

	@Override
    public int getSkipCount() {
		int count = 0;
		for (SanityCheck check : checks.values())
			if (check.getVerdict() == SanityCheckVerdict.SKIPPED)
				count++;
		return count;
	}
	
	@Override
    public int getSuccessCount() {
		int count = 0;
		for (SanityCheck check : checks.values())
			if (check.getVerdict() == SanityCheckVerdict.PASSED)
				count++;
		return count;
	}
	
	@Override
    public int getFailureCount() {
		int count = 0;
		for (SanityCheck check : checks.values())
			if (check.getVerdict() == SanityCheckVerdict.FAILED)
				count++;
		return count;
	}
	
	@Override
    public int getErrorCount() {
		int count = 0;
		for (SanityCheck check : checks.values())
			if (check.getVerdict() == SanityCheckVerdict.ERROR)
				count++;
		return count;
	}

	@Override
    public int getDefectCount() {
		int count = 0;
		for (SanityCheck check : checks.values())
			count += check.getDefectCount();
		return count;
	}
	
	public Date getStartDate() {
		return startDate;
	}
	
	@Override
    public int getExecutionTime() {
		int sum = 0;
		for (SanityCheck check : checks.values())
			sum += check.getExecutionTime();
		return sum;
	}
	
	@Override
	public String toString() {
	    return name;
	}

}
