/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.databene.commons.IOUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.file.FilePrintStream;
import org.databene.dbsanity.DbSanity;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.html.HTMLUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Abstract implementation of the {@link ReportModule} interface.
 * It provides a {@link PrintStream} attribute named <code>out</code>
 * and methods to open and close an HTML file on this attribute.<br/><br/>
 * Created: 13.10.2010 16:39:39
 * @since 1.0
 * @author Volker Bergmann
 */
public abstract class AbstractReportModule implements ReportModule {
	
	protected final Logger logger = LoggerFactory.getLogger(getClass());
	
	protected ReportContext context;
	protected String style = DbSanity.DEFAULT_STYLE;

	public int getDashboardColSpan(ReportScope scope) {
		return 0;
	}
	
	public void setContext(ReportContext context) {
	    this.context = (ReportContext) context;
	}
	
	public void setStyle(String style) {
    	this.style = style;
    }

	public void success(SanityCheck check) {
	}
	
	public void failure(SanityCheck check, Object[] errorInfo, String[] infoHeaders) {
	}
	
	public void error(SanityCheck check, Exception e) {
	}
	
	public void skipped(SanityCheck check) {
	}
	
	public void summary(SanityCheckSuite rootSuite) {
	}
	
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite,
				FilePrintStream out) {
	}
	
	public void close() {
    }
	
	// non-public helpers ----------------------------------------------------------------------------------------------

	protected FilePrintStream openNewFile(File file, String title) throws FileNotFoundException {
		return ReportUtil.openNewFile(file, title, new File(context.getReportRoot(), style + ".css"));
    }

	protected void closeFile(FilePrintStream out) {
		out.println(context.footer());
		out.println("		</center>");
	    out.println("	</body>");
	    out.println("</html>");
	    IOUtil.close(out);
    }

	protected void startModule(FilePrintStream out) {
		out.println("<div class='module'>");
	}
	
	protected void modHeader(String title, FilePrintStream out) {
		out.println("	<div class='mod_header'><h3>" + title + "</h3></div>");
	}

	protected void startModBody(FilePrintStream out) {
		out.println("	<div class='mod_body'>");
	}

	protected void endModBody(FilePrintStream out) {
		out.println("	</div>");
	}

	protected void modFooter(String href, FilePrintStream out) {
		out.println("	<div class='mod_footer'><a href='" + href + "'>Show Report</a></div>");
	}
	
	protected void endModule(FilePrintStream out) {
		out.println("</div>");
	}

	protected void renderCheckInfo(SanityCheck check, FilePrintStream out) {
		startModule(out);
		modHeader("Check Info", out);
		startModBody(out);
		out.println("<table>");
		printNameValueRow("Name;", check.getName(), false, false, out);
		try {
			printNameValueRow("Source File:", check.getSourceFile().getCanonicalPath(), false, false, out);
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
		printNameValueRow("Table:", check.getTable(), false, false, out);
		printNameValueRow("Defect Type:", check.getDefectType(), false, false, out);
		printNameValueRow("Description:", check.getDescription(), true, false, out);
		printNameValueRow("Details:", check.getDetails(), true, true, out);
		printNameValueRow("Author:", check.getAuthor(), true, false, out);
		out.println("</table>");
		endModBody(out);
		endModule(out);
	}
	
	protected void printNameValueRow(String name, String value, boolean dropIfEmpty, boolean pre, FilePrintStream out) {
		if (dropIfEmpty && StringUtil.isEmpty(value))
			return;
		out.println("	<tr>");
		out.print("		<th align='left' valign='top'>" + name +"</th>");
		out.print("		<td align='left'>");
		if (!StringUtil.isEmpty(value)) {
			if (pre)
				out.print("<pre>");
			out.print(HTMLUtil.escape(value));
			if (pre)
				out.print("</pre>");
		}
		out.println("</td>");
		out.println("	</tr>");
	}

}
