/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Date;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.Context;
import org.databene.commons.FileUtil;
import org.databene.commons.context.DefaultContext;
import org.databene.commons.time.ElapsedTimeFormatter;
import org.databene.dbsanity.Version;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.html.HTMLUtil;

/**
 * {@link Context} implementation that provides data and operations for report generation.<br/><br/>
 * Created: 29.11.2010 14:47:03
 * @since 0.4
 * @author Volker Bergmann
 */
public class ReportContext extends DefaultContext {

	public static final String RED    = "EE0000";
	public static final String GREEN  = "00CC00";
	public static final String ORANGE = "EE6600";
	
	private File reportRoot;
	private DecimalFormat dnf;
	private DecimalFormat inf;
	private DateFormat dateFormat;
	private ElapsedTimeFormatter durationFormat;
	private Set<String> tablesToCheck;
	private String ignorableTables;

	public ReportContext(File reportRoot, String[] tables, Locale locale) {
		this.reportRoot = reportRoot;
		this.dnf = new DecimalFormat("#,##0.0", DecimalFormatSymbols.getInstance(locale));
		this.inf = new DecimalFormat("#,##0", DecimalFormatSymbols.getInstance(locale));
		this.dateFormat = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, locale);
		this.durationFormat = new ElapsedTimeFormatter(locale, "&nbsp;");
		this.tablesToCheck = new HashSet<String>();
	}
	
	public File getReportRoot() {
		return reportRoot;
	}
	
	public String formatLong(long number) {
		return inf.format(number);
	}
	
	public String formatDecimal(double number) {
		return dnf.format(number);
	}
	
	public String formatLong(long number, String zeroText) {
		return (number > 0 ? formatLong(number) : zeroText);
	}

	public String formatDuration(long executionTime) {
		return durationFormat.convert(executionTime);
	}

	public String formatDate(Date date) {
		return (date != null ? dateFormat.format(date) : "");
	}

	public boolean acceptsTable(String tableName) {
		if (tablesToCheck.isEmpty())
			return true; // no filter set
		return (tableName != null && tablesToCheck.contains(tableName));
	}

	public String navBar(File fromFolder, String... refLabelPairs) {
		String dashboardRef = FileUtil.relativePath(fromFolder, reportRoot, '/') + "/index.html";
		String result = "<div class='nav_bar'><table width='100%'><tr>" +
					"<td align='left'>" + HTMLUtil.a(dashboardRef, "Dashboard");
		for (int i = 0; i < refLabelPairs.length; i += 2)
			result += " | <a href='" + refLabelPairs[i] + "'>" + refLabelPairs[i + 1] + "</a></td>";
		result += 	"</td>" +
					"<td align='right'><a href='http://databene.org/dbsanity' target='_blank'>Help</a></td>" +
				"</tr></table></div>";
		return result;
	}

	public String verdictImage(SanityCheckVerdict verdict, File fromFile) {
		String verdictName = verdict.name().toLowerCase();
		String fileName = verdictName + "-16.gif";
		File imgFile = new File(reportRoot, fileName);
		return "<img src='" + FileUtil.relativePath(fromFile, imgFile, '/') + "' alt='" + verdictName + "'/>";
	}

	public Set<String> getTablesToCheck() {
		return tablesToCheck;
	}
	
	public void setTablesToCheck(String[] tablesToCheck) {
		this.tablesToCheck = CollectionUtil.toSet(tablesToCheck);
	}

	public String getIgnorableTables() {
		return ignorableTables;
	}
	
	public void setIgnorableTables(String ignorableTables) {
		this.ignorableTables = ignorableTables;
	}
/*
	public String header(File reportFile, String title) {
		File logoFile = new File(reportRoot, "dbsanity-logo-64.gif");
		return "<table border='0'><tr>" +
				"<td><img src='" + FileUtil.relativePath(reportFile, logoFile, '/') + "' border='0' alt='DB Sanity Logo'/></td>" +
				"<td style='vertical-align:middle;'><h1>" + title + "</h1></td>" +
				"</tr></table>";
	}
*/	
	public String footer() {
		return "			<p class='orange'><b>Report generated by Volker Bergmann's <a href='http://databene.org' target='_blank' style='color:#AA4400;'>Databene</a> <a href='http://databene.org/dbsanity' target='_blank' style='color:#AA4400;'>DB Sanity " + Version.VERSION + "</a></b></p>";
	}

}
