/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.databene.commons.FileUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.StringCharacterIterator;
import org.databene.commons.SystemInfo;
import org.databene.commons.file.FilePrintStream;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.SanityCheckFolder;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.html.HTMLUtil;

/**
 * Provides utility methods for report generation.<br/><br/>
 * Created: 14.10.2010 16:14:08
 * @since 1.0
 * @author Volker Bergmann
 */
public class ReportUtil { // TODO v0.7 move methods to ReportContext
	
	public static String normalizeUrl(String rawUrl) {
		StringBuffer url = new StringBuffer(rawUrl.length());
		StringCharacterIterator iterator = new StringCharacterIterator(rawUrl);
		while (iterator.hasNext()) {
			char c = iterator.next();
			if (Character.isLetterOrDigit(c) || c=='.' || c == '-'|| c == '*' || c == '_'|| c == '+')
				url.append(c);
			else if (c == ' ')
				url.append('_');
			// all other characters are ignored
		}
		return url.toString();
	}
	
	public static void summaryBar(SanityCheckSuite suite, String width, FilePrintStream out) {
		out.println("	<table width='" + width + "' cellpadding='0' cellspacing='0'>");
		out.println("	<tr align='left'>");
		if (suite.getErrorCount() > 0) {
			out.println("		<td width='"
					+ getErrorPercentage(suite)
					+ "%'><div style='background: #000000; color: #ffffff;' title='Tests with errors'>" + suite.getErrorCount() + "</div></td>");
		}
		if (suite.getFailureCount() > 0) {
			out.println("		<td width='"
					+ getFailurePercentage(suite)
					+ "%'><div style='background: #" + ReportContext.RED + "; color: #ffffff;' title='Failed tests'>" + suite.getFailureCount() + "</div></td>");
		}
		if (suite.getSuccessCount() > 0) {
			out.println("		<td width='"
					+ getSuccessPercentage(suite)
					+ "%'><div style='background: #" + ReportContext.GREEN + "; color: #ffffff;' title='Passed tests'>" + suite.getSuccessCount() + "</div></td>");
		}
		if (suite.getSkipCount() > 0) {
			out.println("		<td width='"
					+ getSkipPercentage(suite)
					+ "%'><div style='background: #bbb; color: #ffffff;' title='Skipped tests'>" + suite.getSkipCount() + "</div></td>");
		}
		out.println("	</tr>");
		out.println("	</table>");
	}

	public static void numberBar(long n, String width, String color, ReportContext context, FilePrintStream out) {
		bar(n, context.formatLong(n), width, color, context, out);
	}

	public static void durationBar(long n, String width, String color, ReportContext context, FilePrintStream out) {
		bar(n, context.formatDuration(n), width, color, context, out);
	}

	public static void bar(long n, String label, String width, String color, ReportContext context, FilePrintStream out) {
		out.println("	<table width='" + width + "' cellpadding='0' cellspacing='0'>");
		out.println("	<tr>");
		if (n > 0)
			out.println("		<td width='100%' align='left'><div style='background: " + color +"; color: #ffffff; font-size: small'>" + label + "</div></td>");
		out.println("	</tr>");
		out.println("	</table>");
	}

	public static double getSkipPercentage(SanityCheckSuite suite) {
		if (getTestCount(suite) > 0)
			return suite.getSkipCount() * 100. / getTestCount(suite);
		else return 0;
	}

	public static double getSuccessPercentage(SanityCheckSuite suite) {
		if (getTestCount(suite) > 0)
			return suite.getSuccessCount() * 100. / getTestCount(suite);
		else return 0;
	}

	public static double getFailurePercentage(SanityCheckSuite suite) {
		if (getTestCount(suite) > 0)
			return suite.getFailureCount() * 100. / getTestCount(suite);
		else return 0;
	}

	public static double getErrorPercentage(SanityCheckSuite suite) {
		if (getTestCount(suite) > 0)
			return suite.getErrorCount() * 100 / getTestCount(suite);
		else
			return 0;
	}

	public static int getTestCount(SanityCheckSuite suite) {
		return suite.getSuccessCount() 
				+ suite.getFailureCount()
				+ suite.getErrorCount()
				+ suite.getSkipCount();
	}

	public static String docLink(SanityCheck check, File fromFile) {
		String link = docRef(check, fromFile);
		return "<a href='" + link + "'>" + check.getName() + "</a>";
	}
	
	public static String docLink(SanityCheck check, File fromFile, String label) {
		return "<a href='" + docRef(check, fromFile) + "'>" + label + "</a>";
	}

	public static String docRef(SanityCheck check, File fromFile) {
		return FileUtil.relativePath(fromFile, check.getDocPage(), '/') + '#' + check.getName();
	}
	
	public static List<SanityCheck> agregateChecks(SanityCheckSuite suite, boolean recursive) {
		return aggregateChecks(suite, new ArrayList<SanityCheck>(), recursive);
	}

	public static void copyResourceFiles(File targetFolder, String style) throws IOException {
		FileUtil.ensureDirectoryExists(targetFolder);
		copyResourceFile("dbsanity-logo-64.gif", targetFolder);
		copyResourceFile("pending-16.gif", targetFolder);
		copyResourceFile("passed-16.gif", targetFolder);
		copyResourceFile("failed-16.gif", targetFolder);
		copyResourceFile("error-16.gif", targetFolder);
		copyResourceFile("skipped-16.gif", targetFolder);
		copyResourceFile("favicon.ico", targetFolder);
		copyResourceFile(style + ".css", targetFolder);
	}

	public static void copyResourceFile(String filename, File folder) throws IOException {
		IOUtil.copyFile(filename, folder.getAbsolutePath() + SystemInfo.getFileSeparator() + filename);
	}
	
	public static String docOrDefectFileLink(SanityCheck check, File fromFile) {
		return "<a href='" + docOrDefectFileRef(check, fromFile) + "'>" + check.getName() + "</a>";
	}

	public static String docOrDefectFileRef(SanityCheck check, File fromFile) {
		String link;
		SanityCheckVerdict verdict = check.getVerdict();
		if (verdict == SanityCheckVerdict.ERROR || verdict == SanityCheckVerdict.FAILED)
			link = FileUtil.relativePath(fromFile, check.getDefectsOrErrorPage(), '/');
		else {
			File docPage = check.getDocPage();
			link = docPage.getParentFile().getName() + "/" + docPage.getName() + "#" + check.getName();
		}
		return link;
	}

	public static FilePrintStream openNewFile(File file, String title, File cssFile) throws FileNotFoundException {
		FilePrintStream out = new FilePrintStream(file);
	    out.println(HTMLUtil.DOCTYPE_401);
	    out.println("<html>");
	    out.println("	<head>");
	    out.println("		<title>" + title + "</title>");
	    String cssRef = FileUtil.relativePath(file, cssFile, '/');
	    out.println("		<meta http-equiv='Content-Type' content='text/html; charset=" + SystemInfo.getFileEncoding() + "' />");
	    out.println("		<link href='" + cssRef + "' media='all' rel='stylesheet' type='text/css' />");
	    out.println("	</head>");
	    out.println("	<body>");
	    out.println("		<div class='page_header'>");
	    out.println("			<h1>" + title + "</h1>");
	    out.println("		</div>");
	    out.println("		<center>");
	    return out;
    }

	// private helpers -------------------------------------------------------------------------------------------------
	
	private static List<SanityCheck> aggregateChecks(SanityCheckSuite suite,
			List<SanityCheck> checks, boolean recursive) {
		if (recursive && suite instanceof SanityCheckFolder)
			for (SanityCheckSuite childSuite : ((SanityCheckFolder) suite).getChildSuites().values())
				aggregateChecks(childSuite, checks, recursive);
		if (suite instanceof SanityCheckFile)
			checks.addAll(((SanityCheckFile) suite).getChecks().values());
		return checks;
	}
	
}
